/*
 * Include the Control Surface Library
 */
#include <Control_Surface.h>

/*
 * Use USB as the interface for MIDI communication.
 */
USBMIDI_Interface midi;

/*
 * Create a custom sender that marries an LED with a specific push button. It is
 * created as a sub-class to DigitalCCSender that is part of the library
 */
class LEDFeedbackDigiCCSender: public DigitalCCSender
{
	public:
		/*
		 * Constructor, that additionally takes in the pin for the associated
		 * LED.
		 */
		LEDFeedbackDigiCCSender(pin_t ledPin)
			: DigitalCCSender(), 
			ledPin(ledPin),
			onState(false)
		{
		}

		/*
		 * When the address needs to be sent an ON CC message.
		 */
		void sendOn(MIDIAddress address)
		{
			DigitalCCSender::sendOn(address);
		}

		/*
		 * Every button push of a momentary switch is two actions - an ON when
		 * pushed, and an OFF when released. We are essentially only controlling
		 * the LED when the button is released.
		 */
		void sendOff(MIDIAddress address)
		{
			/*
			 * First send the OFF message through MIDI.
			 */
			DigitalCCSender::sendOff(address);

			/*
			 * Now check what the previously known status of the LED is. If it
			 * was known to be on, then turn it off, it was known to be on, then
			 * turn in on.
			 */
			if(onState)
			{
				digitalWrite(ledPin, LOW);
			}
			else
			{
				digitalWrite(ledPin, HIGH);
			}
			onState = !onState;
		}

	private:
		pin_t ledPin;
		boolean onState;
};

/*
 * In order to use the custom listener, we are also creating a custom button.
 * The button is essentially a pair of digital devices - a push button, and an
 * associated LED. This is there a MIDIButton templated with the custom Sender
 * we just created.
 */
struct CustomDigiButton : MIDIButton<LEDFeedbackDigiCCSender>
{
	/*
	 * Constructor - create the MIDIButton with the custom sender initiated, and
	 * also initialise the LED to off.
	 */
	CustomDigiButton(pin_t pin, MIDIAddress address, pin_t ledpin)
		: MIDIButton(pin, address, {ledpin})
	{
		/*
		 * FIXME: As a design principle, perhaps this should be in the Sender
		 * constructor and not in this constructor, as the sender is the one
		 * otherwise altering the state of the LED.
		 */
		digitalWrite(ledpin, LOW);
	}
};

/*
 * Initialise all the potentiometers.
 * There are six different modules attached to the main arduino.
 */
CCPotentiometer potentiometers[]
{
	// board 1 - three pots
	{A0, 1}, // Pin number of first pot, Controller number of the first potentiometer
	{A1, 2}, // Pin number of second pot, Controller number of the second potentiometer
	{A2, 3}, 
	//board 2 - three pots
	{A3, 4},
	{A4, 5},
	{A5, 6}, 
	// board 3 - two pots
	{A6, 7}, 
	{A7, 8},
	 // board 4 - three pots - main amp volume and gain
	{A8, 9},
	{A9, 10},
	{A10, 11},
	// board 5 - two pots
	{A11, 12},
	{A12, 13},
	// board 6 - two pots, one of which is an expression pedal
	{A13, 14},
	{A14, 15},
};

/*
 * Setup the custom digital push buttons. Recall, they are associated with a
 * companion LED.
 */
CustomDigiButton buttons []
{
	{13, 21, 12}, // push button on pin 13, CC # 21, LED pin 12
	{11, 22, 10},
	{9, 23, 8},
	{7, 24, 6},
	{5, 25, 4},
};

/*
 * This is just the out of the box standard Control_Surface setup
 */
void setup()
{
	Control_Surface.begin(); // Initialize Control Surface
}

/*
 * Standard Control_Surface loop.
 */
void loop()
{
	Control_Surface.loop(); // Update the Control Surface
}
